#include <wx/app.h>
#include <wx/frame.h>
#include <wx/string.h>
#include <wx/panel.h>
#include <wx/sizer.h>
#include <wx/menu.h>
#include <wx/filepicker.h>
#include <wx/filectrl.h>
#include <wx/stattext.h>
#include <wx/checkbox.h>
#include <cstdlib>

#ifndef GUI_BASE_H
#define GUI_BASE_H
 
// MainApp is the class for our application, it just acts
// as a container for the window or frame in MainFrame.
class MainApp: public wxApp {
    public:
        virtual bool OnInit();
};
 
// MainFrame is the class for our window, it
// contains the window and all objects in it.
class MainFrame: public wxFrame {
    public:
        MainFrame( const wxString &title, const wxPoint &pos, const wxSize &size );

        wxMenuBar *MainMenu;
        wxSizer *Sizer;

        void Quit(wxCommandEvent &event);
        DECLARE_EVENT_TABLE()
};

class MainPanel: public wxPanel {
    public:
        MainPanel(wxWindow *parent, wxWindowID id=wxID_ANY, const wxPoint &pos=wxDefaultPosition, const wxSize &size=wxDefaultSize);
        void OnFileChange(wxFileDirPickerEvent &event);
        void OnButtonPress(wxCommandEvent &event);
        static bool validateString(const std::string, const std::string);

        wxStaticText *inputLabel;
        wxFilePickerCtrl *inputFileCtrl;
        wxStaticText *colorLabel;
        wxFilePickerCtrl *colorFileCtrl;
        wxStaticText *outputLabel;
        wxFilePickerCtrl *outputFileCtrl;
        // The Path to the file we have open
        wxString CurrentDocPath;
        wxSizer *inputSizer;
        wxSizer *colorSizer;
        wxSizer *outputSizer;
};

enum
{
	// declares an id which will be used to call our button
    LEFT_Panel,
    RIGHT_Panel,
	LFILE_Input,
    LFILE_Color,
    LFILE_Output,
    LTEXT_Input,
    LTEXT_Color,
    LTEXT_Output,
	RFILE_Input,
    RFILE_Color,
    RFILE_Output,
    RTEXT_Input,
    RTEXT_Color,
    RTEXT_Output,
    NODE_Button,
    EDGE_Button,
	MENU_Quit
};
 
BEGIN_EVENT_TABLE (MainFrame, wxFrame)
	EVT_MENU(MENU_Quit, MainFrame::Quit)
    EVT_FILEPICKER_CHANGED(LFILE_Input, MainPanel::OnFileChange)
    EVT_FILEPICKER_CHANGED(LFILE_Color, MainPanel::OnFileChange)
    EVT_FILEPICKER_CHANGED(LFILE_Output, MainPanel::OnFileChange)
    EVT_FILEPICKER_CHANGED(RFILE_Input, MainPanel::OnFileChange)
    EVT_FILEPICKER_CHANGED(RFILE_Color, MainPanel::OnFileChange)
    EVT_FILEPICKER_CHANGED(RFILE_Output, MainPanel::OnFileChange)
    EVT_BUTTON(NODE_Button, MainPanel::OnButtonPress)
    EVT_BUTTON(EDGE_Button, MainPanel::OnButtonPress)
END_EVENT_TABLE()

#endif /* ifndef GUI_BASE_H */
