#include <wx/wxprec.h>
#include <iostream>
#include <wx/font.h>
#include <wx/msgdlg.h>
#include "colored-gdv.h"
using namespace std;
 
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif
 
#include "base.h"
 
IMPLEMENT_APP(MainApp) // Initializes the MainApp class...
 
// .. and tells our program to run it
bool MainApp::OnInit()
{
	// Create an instance of our frame, or window
	MainFrame *MainWin = new MainFrame(
		_("Colored Graphlets"), wxPoint(1, 1), wxSize(1000, 500));
    MainWin->SetMinSize(wxSize(1000, 500));
    MainWin->SetMaxSize(wxSize(1000, 500));
	MainWin->Show(TRUE); // show the window
	SetTopWindow(MainWin); // and finally, set it as the main window
	return TRUE;
}
 
MainFrame::MainFrame(const wxString &title, const wxPoint &pos, const wxSize &size)
: wxFrame((wxFrame*) NULL, -1, title, pos, size) {
    int width = 0;
    int height = 0;
    this->GetSize(&width, &height);
    cout << width << endl;
    cout << height << endl;

    MainPanel *leftPanel = new MainPanel(this, LEFT_Panel, wxPoint(0,0), wxSize(width/2, 450));
    leftPanel->Show(TRUE);
    MainPanel *rightPanel = new MainPanel(this, RIGHT_Panel, wxPoint(width/2, 0), wxSize(width/2, 450));
    rightPanel->Show(TRUE);

    Sizer = new wxBoxSizer(wxVERTICAL);
    wxSizer *hSizer = new wxBoxSizer(wxHORIZONTAL);
    hSizer->Add(leftPanel, .5, wxALIGN_TOP|wxRIGHT, 5);
    hSizer->Add(rightPanel, .5, wxALIGN_TOP|wxLEFT, 5);
    Sizer->Add(hSizer, 1, wxEXPAND|wxTOP|wxRIGHT|wxLEFT, 10);

    this->SetSizer(Sizer);
}

void MainFrame::Quit(wxCommandEvent &WXUNUSED(event)) {
	Close(TRUE); // Close the window
}

MainPanel::MainPanel(wxWindow *parent, wxWindowID id, const wxPoint &pos, const wxSize &size) : wxPanel (parent, id, pos, size, wxBORDER_SUNKEN) {
    int TEXT_WIDTH = 200;
    int TEXT_HEIGHT = 24;
    int TITLE_WIDTH = 300;
    int TITLE_HEIGHT = 32;

	inputLabel = new wxStaticText(
		this, id == LEFT_Panel ? LTEXT_Input : RTEXT_Input, "Input Network File", wxDefaultPosition, wxSize(TEXT_WIDTH, TEXT_HEIGHT));
    inputFileCtrl = new wxFilePickerCtrl(
            this, id == LEFT_Panel ? LFILE_Input : RFILE_Input, wxEmptyString, "Select a file", wxFileSelectorDefaultWildcardStr);
    colorLabel = new wxStaticText(
            this, id == LEFT_Panel ? LTEXT_Color : RTEXT_Color, id == LEFT_Panel ? "Node Color File" : "Edge Color File", wxDefaultPosition, wxSize(TEXT_WIDTH, TEXT_HEIGHT));
    colorFileCtrl = new wxFilePickerCtrl(
            this, id == LEFT_Panel ? LFILE_Color : RFILE_Color, wxEmptyString, "Select a file", wxFileSelectorDefaultWildcardStr);
    outputLabel = new wxStaticText(
            this, id == LEFT_Panel ? LTEXT_Output : RTEXT_Output, "Output File", wxDefaultPosition, wxSize(TEXT_WIDTH, TEXT_HEIGHT));
    outputFileCtrl = new wxFilePickerCtrl(
            this, id == LEFT_Panel ? LFILE_Output : RFILE_Output, wxEmptyString, "Select a file", wxFileSelectorDefaultWildcardStr, wxDefaultPosition, wxDefaultSize, wxFC_SAVE);

    wxFont labelFont(TEXT_HEIGHT/2, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    inputLabel->SetFont(labelFont);
    colorLabel->SetFont(labelFont);
    outputLabel->SetFont(labelFont);

    wxButton *runButton = new wxButton(this, id == LEFT_Panel ? NODE_Button : EDGE_Button, "Run");

    wxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);
    wxSizer *titleSizer = new wxBoxSizer(wxHORIZONTAL);
    inputSizer = new wxBoxSizer(wxHORIZONTAL);
    colorSizer = new wxBoxSizer(wxHORIZONTAL);
    outputSizer = new wxBoxSizer(wxHORIZONTAL);

    wxStaticText *titleLabel = new wxStaticText(
            this, wxID_ANY, id == LEFT_Panel ? "Node Colored Graphlets" : "Edge Colored Graphlets", wxDefaultPosition, wxSize(TITLE_WIDTH, TITLE_HEIGHT));
    wxFont titleFont(TITLE_HEIGHT/2, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD);
    titleLabel->SetFont(titleFont);
    titleSizer->Add(titleLabel, 0, wxALIGN_CENTER, 0);

    inputSizer->Add(inputLabel, 1, wxALIGN_CENTER_VERTICAL|wxALL, 10);
    inputSizer->Add(inputFileCtrl, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 10);
    colorSizer->Add(colorLabel, 1, wxALIGN_CENTER_VERTICAL|wxALL, 10);
    colorSizer->Add(colorFileCtrl, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 10);
    outputSizer->Add(outputLabel, 1, wxALIGN_CENTER_VERTICAL|wxALL, 10);
    outputSizer->Add(outputFileCtrl, 1, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 10);

    mainSizer->Add(titleSizer, 0, wxALIGN_CENTER|wxALIGN_TOP|wxTOP, 20);
    mainSizer->Add(inputSizer, 0,  wxALIGN_TOP|wxTOP, 20);
    mainSizer->Add(colorSizer, 0, wxALIGN_TOP|wxALL, 0);
    mainSizer->Add(outputSizer, 0, wxALIGN_TOP|wxALL, 0);
    mainSizer->Add(runButton, 0, wxALIGN_CENTER|wxALL, 20);

    if (id == RIGHT_Panel) {
        mainSizer->Show(colorSizer, FALSE);
    }

    this->SetSizer(mainSizer);
}

void MainPanel::OnFileChange(wxFileDirPickerEvent &event) {
}

void MainPanel::OnButtonPress(wxCommandEvent &event) {
    wxButton *button = (wxButton *) FindWindowById(event.GetId());
    button->Enable(FALSE);
    button->SetLabel("Running...");
    switch (event.GetId()) {
        case EDGE_Button:
            {
                string input = ((wxFilePickerCtrl *)FindWindowById(RFILE_Input))->GetPath().ToStdString();
                string output = ((wxFilePickerCtrl *)FindWindowById(RFILE_Output))->GetPath().ToStdString();
                bool valid = MainPanel::validateString(input, "Input file cannot be empty") ||
                    MainPanel::validateString(output, "Output file cannot be empty");
                if (valid) {
                    int argc = 4;
                    char** argv = new char*[4];
                    argv[0] = strdup("gui");
                    argv[1] = strdup("-edge");
                    argv[2] = strdup(input.c_str());
                    argv[3] = strdup(output.c_str());
                    try {
                        compute_gdv(argc, argv);
                        wxMessageBox(wxT("Output saved to " + output));
                    } catch(exception &e) {
                        wxMessageBox(e.what());
                    }
                }
            }
            break;
        case NODE_Button:
            {
                string input = ((wxFilePickerCtrl *)FindWindowById(LFILE_Input))->GetPath().ToStdString();
                string colors = ((wxFilePickerCtrl *)FindWindowById(LFILE_Color))->GetPath().ToStdString();
                string output = ((wxFilePickerCtrl *)FindWindowById(LFILE_Output))->GetPath().ToStdString();
                bool valid = MainPanel::validateString(input, "Input file cannot be empty") ||
                    MainPanel::validateString(colors, "Color file cannot be empty") ||
                    MainPanel::validateString(output, "Output file cannot be empty");
                if (valid) {
                    int argc = 5;
                    char** argv = new char*[5];
                    argv[0] = strdup("gui");
                    argv[1] = strdup("-node");
                    argv[2] = strdup(input.c_str());
                    argv[3] = strdup(colors.c_str());
                    argv[4] = strdup(output.c_str());
                    try {
                        compute_gdv(argc, argv);
                        wxMessageBox(wxT("Output saved to " + output));
                    } catch(exception &e) {
                        wxMessageBox(e.what());
                    }
                }
            }
            break;
    }
    button->SetLabel("Run");
    button->Enable(TRUE);
}

bool MainPanel::validateString(const std::string s, const std::string message) {
    if (!s.length()) {
        wxMessageBox(message);
        return false;
    }

    return true;
}
